module doodle.dia.grid_layer;

public {
    import doodle.dia.icanvas;
}

private {
    import doodle.cairo.routines;
    import std.math;
    import std.stdio;
}

private {
    double start(in double value, in double spacing) {
        real r = floor(value / spacing);
        return r * spacing;
    }
}

class GridLayer : Layer, Grid {
    static const double MIN_SPACING = 5.0;      // pixels

    this(in string name) {
        super(name);
    }

    // Layer overrides:

    override Rectangle bounds() const {
        // We don't require any geometry
        return Rectangle();
    }

    override void draw(in Viewport viewport,
                       in Rectangle pixel_damage, scope Context pixel_cr,
                       in Rectangle model_damage, scope Context model_cr) const {
        assert(mZoomValid);

        model_cr.save(); {
            model_cr.setLineWidth(1.0);

            {
                // vertical grid lines
                double x = start(model_damage.min_corner.x, mSpacing);
                double y0 = model_damage.min_corner.y;
                double y1 = model_damage.max_corner.y;

                for (;;) {
                    line(model_cr, x, y0, x, y1);
                    model_cr.stroke();

                    if (x > model_damage.max_corner.x) {
                        break;
                    }

                    x += mSpacing;
                }
            }

            {
                // horizontal grid lines
                double y = start(model_damage.min_corner.y, mSpacing);
                double x0 = model_damage.min_corner.x;
                double x1 = model_damage.max_corner.x;

                for (;;) {
                    line(model_cr, x0, y, x1, y);
                    model_cr.stroke();

                    if (y > model_damage.max_corner.y) {
                        break;
                    }

                    y += mSpacing;
                }
            }
        } model_cr.restore();


        //double start_x = modf(damage.min_corner.x, zoom);
    }

    // Grid overrides:

    void zoom_changed(double zoom) {
        mZoom = zoom;
        mZoomValid = true;
        mSpacing = 20.0;        // mm
    }

    bool snap(in Point a, out Point b) const {
        b = a;
        return false;
    }

    private {
        bool mZoomValid;
        double mZoom;           // pixels per millimetre

        double mSpacing;        // model spacing
    }
}
